
From: Keshavamurthy Anil S <anil.s.keshavamurthy@intel.com>

Once the jprobe instrumented function returns, it executes a jprobe_break
which is a break instruction with __IA64_JPROBE_BREAK value.  The current
patch checks for this break value, before assuming that jprobe instrumented
function just completed.

The previous code was not checking for this value and that was a bug.

Signed-off-by: Anil S Keshavamurthy <anil.s.keshavamurthy@intel.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 arch/ia64/kernel/kprobes.c |   10 +++++++---
 1 files changed, 7 insertions(+), 3 deletions(-)

diff -puN arch/ia64/kernel/kprobes.c~kprobes-ia64-check-jprobe-break-before-handling arch/ia64/kernel/kprobes.c
--- 25/arch/ia64/kernel/kprobes.c~kprobes-ia64-check-jprobe-break-before-handling	2005-06-18 02:49:06.000000000 -0700
+++ 25-akpm/arch/ia64/kernel/kprobes.c	2005-06-18 02:49:06.000000000 -0700
@@ -419,10 +419,11 @@ static void prepare_ss(struct kprobe *p,
 	ia64_psr(regs)->ss = 1;
 }
 
-static int pre_kprobes_handler(struct pt_regs *regs)
+static int pre_kprobes_handler(struct die_args *args)
 {
 	struct kprobe *p;
 	int ret = 0;
+	struct pt_regs *regs = args->regs;
 	kprobe_opcode_t *addr = (kprobe_opcode_t *)instruction_pointer(regs);
 
 	preempt_disable();
@@ -437,7 +438,7 @@ static int pre_kprobes_handler(struct pt
 			}
 			arch_disarm_kprobe(p);
 			ret = 1;
-		} else {
+		} else if (args->err == __IA64_BREAK_JPROBE) {
 			/*
 			 * jprobe instrumented function just completed
 			 */
@@ -445,6 +446,9 @@ static int pre_kprobes_handler(struct pt
 			if (p->break_handler && p->break_handler(p, regs)) {
 				goto ss_probe;
 			}
+		} else {
+			/* Not our break */
+			goto no_kprobe;
 		}
 	}
 
@@ -515,7 +519,7 @@ int kprobe_exceptions_notify(struct noti
 	struct die_args *args = (struct die_args *)data;
 	switch(val) {
 	case DIE_BREAK:
-		if (pre_kprobes_handler(args->regs))
+		if (pre_kprobes_handler(args))
 			return NOTIFY_STOP;
 		break;
 	case DIE_SS:
_
